-- Some constants for configuration
local DIST_TO_FRAME = 10
local DIST_TO_ICON = 5
local FONT_SIZE = 16



Gladiminish = LibStub("AceAddon-3.0"):NewAddon("Gladiminish", "AceEvent-3.0", "AceHook-3.0")
local GladiusAddon

function Gladiminish:Print(...)
	local text = ""
	text = "|cff33ff99"..tostring( self ).."|r: "

	for i=1, select("#", ...) do
		text = text .. tostring( select( i, ...) ) .." "
	end
	DEFAULT_CHAT_FRAME:AddMessage( text )
end

function Gladiminish:OnInitialize()
	self.DRData = LibStub("DRData-1.0")
	if self.DRData then
		-- Nature's Grasp
		self.DRData.spells[19970] = "root"
		self.DRData.spells[19971] = "root"
		self.DRData.spells[19972] = "root"
		self.DRData.spells[19973] = "root"
		self.DRData.spells[19974] = "root"
		self.DRData.spells[19975] = "root"
		self.DRData.spells[27010] = "root"
		self.DRData.spells[53313] = "root"
	end
	
	self.trackedGUID = {}
	self.iFrames = {}
	self.unusedIcons = {}
end

function Gladiminish:OnEnable()
	if Gladius and self.DRData then
		GladiusAddon = Gladius
		
		self:SecureHook(GladiusAddon, "JoinedArena", self.JoinedArena)
		self:SecureHook(GladiusAddon, "Test", self.UpdateiFrames)
		self:SecureHook(GladiusAddon, "UpdateFrame", self.UpdateiFrames)
		
		self:SecureHook(GladiusAddon, "LeftArena", self.ClearAllFrames)
		self:SecureHook(GladiusAddon, "UpdateUnit", self.UpdateUnit)

		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end

function Gladiminish:OnDisable()
	self:UnHookAll()
	self:UnRegisterAllEvents()
	
	table.wipe(self.trackedGUID)
end

-- Steal UnitGUIDs from Gladius
function Gladiminish:UpdateUnit(unit)
	if unit and GladiusAddon.buttons[unit].GUID then
		Gladiminish.trackedGUID[GladiusAddon.buttons[unit].GUID] = unit
	end
end

-- Clear old data
function Gladiminish:ClearAllFrames()
	local self = Gladiminish
	
	for guid, unit in pairs(self.trackedGUID) do
		self:ClearAllDR(unit)
		self.trackedGUID[guid] = nil
	end
end

-- Update iFrames before every match and after the Gladius frame changed
function Gladiminish:UpdateiFrames()
	local self = Gladiminish
	local bracket = GladiusAddon.currentBracket or 5
	
	for i=1, bracket do
		local name = "arena"..i
		if not self.iFrames[name] then
			self.iFrames[name] = self:CreateIconFrame(i)	
		else
			self:UpdateIconFrame(i)
		end
		
		self.iconWidth = self.iFrames[name]:GetHeight()
		if GladiusAddon.buttons[name].GUID then
			self.trackedGUID[GladiusAddon.buttons[name].GUID] = name
		end
	end	
	
	-- The first frame doesn't get positioned correctly, so it needs to be adjusted
	local iFrame = Gladiminish.iFrames["arena1"]
	if iFrame then
		local diff = (GladiusButton1:GetLeft() or 0) - (GladiusButtonFrame1:GetLeft() or 0)
	
		iFrame:ClearAllPoints()
		iFrame:SetPoint("TOPRIGHT", "GladiusButton1", "TOPLEFT", -DIST_TO_FRAME - diff, 0)
		iFrame:SetPoint("BOTTOMRIGHT", "GladiusButton1", "BOTTOMLEFT", -DIST_TO_FRAME - diff, 0)
	end
end

-- Also clear data when joining arena (could have been in test mode)
function Gladiminish:JoinedArena()
	Gladiminish:ClearAllFrames()
	Gladiminish:UpdateiFrames()
end

local trackedEvents = { ["SPELL_AURA_REMOVED"] = true, ["SPELL_AURA_APPLIED"] = true, ["SPELL_AURA_REFRESH"] = true, ["PARTY_KILL"] = true }
function Gladiminish:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellID, spellName, spellSchool, auraType)
	if self.trackedGUID[destGUID] and trackedEvents[eventType] then
	
		if ( eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESH" ) then
			if ( auraType == "DEBUFF" and self.DRData:GetSpellCategory(spellID) ) then
				--self:Print("Gained:",self.trackedGUID[destGUID], spellID, spellName)
				self:DebuffGained(self.trackedGUID[destGUID], spellID)
			end
		elseif eventType == "SPELL_AURA_REMOVED" then
			if ( auraType == "DEBUFF" and self.DRData:GetSpellCategory(spellID) ) then
				--self:Print("Faded:",self.trackedGUID[destGUID], spellID, spellName)
				self:DebuffFaded(self.trackedGUID[destGUID], spellID)
			end
		elseif eventType == "PARTY_KILL" then
			self:ClearAllDR(self.iFrames[self.trackedGUID[destGUID]])
		end
	end
end

-- Create a DR item, but fix its duration
function Gladiminish:DebuffGained(unit, spellID)
	local category = self.DRData:GetSpellCategory(spellID)
	local duration = self.DRData:GetResetTime()
	
	-- Is there already a diminishing return?
	for i, icon in pairs(self.iFrames[unit].icons) do
		if icon.category == category then
			self:RefreshIcon(icon, self.iFrames[unit], category, spellID, self.DRData:NextDR(icon.factor), duration)
			icon.fixed = true
			self:PositionIcons()
			return
		end
	end
	
	-- Create a new diminishing return
	local icon
	if (#Gladiminish.unusedIcons > 0) then
		icon = table.remove(Gladiminish.unusedIcons)
		Gladiminish:RefreshIcon(icon, self.iFrames[unit], category, spellID, 0.5, duration)
	else
		icon = Gladiminish:CreateIcon(self.iFrames[unit], category, spellID, 0.5, duration)
	end
	icon.fixed = true
	
	table.insert(self.iFrames[unit].icons, icon)
	self.iFrames[unit].numIcons = #self.iFrames[unit].icons		

	self:PositionIcons()
	icon:Show()
end

-- Find DR icon and unfix it
function Gladiminish:DebuffFaded(unit, spellID)
	local category = self.DRData:GetSpellCategory(spellID)
	
	for i, icon in pairs(self.iFrames[unit].icons) do
		if icon.category == category then
			icon.fixed = false
			self:PositionIcons()
			break
		end
	end
end

-- Clear every DR of a given unit
function Gladiminish:ClearAllDR(unit)
	local self = Gladiminish
	local iFrame = self.iFrames[unit]
	
	if iFrame then
		while #iFrame.icons > 0 do
			local icon = table.remove(iFrame.icons)
			icon:Hide()
			table.insert(self.unusedIcons, icon)
		end
		
		self:PositionIcons()
	end
end

-- Sort and position the icons
function Gladiminish:PositionIcons()
	for _, iFrame in pairs(self.iFrames) do
		local side = iFrame:GetHeight()
		iFrame:SetWidth(#iFrame.icons * ( side + 10 ) + 10)

		table.sort(iFrame.icons, function(a, b)
									if a.fixed ~= b.fixed then
										return b.fixed
									else
										return a.timeLeft < b.timeLeft
									end
								end)
		for i, icon in ipairs(iFrame.icons) do
			icon:ClearAllPoints()
			icon:SetPoint("TOPRIGHT", iFrame, "TOPRIGHT", -(i - 1) * ( side + DIST_TO_ICON ), 0)
		end
	end
end

-- Update displayed icons in an iFrame
local function iconUpdate(self, elapsed)
	if (self.numIcons > 0) then
		for i, icon in pairs(self.icons) do
			if not icon.fixed then
				icon.timeLeft = icon.timeLeft - elapsed
			end
			
			if icon.timeLeft >= 10 then
				icon.durationText:SetFormattedText("%.0f", icon.timeLeft)
			elseif icon.timeLeft > 0 then
				icon.durationText:SetFormattedText("%.1f", icon.timeLeft)
			else
				self.icons[i]:Hide()
				table.insert(Gladiminish.unusedIcons, table.remove(self.icons, i))
				
				Gladiminish:PositionIcons()	
			end
		end
	end
end

-- Create an iconframe and position it next to the Gladius Frame
function Gladiminish:CreateIconFrame(i)
	local gladiusButton = "GladiusButton"..i
	local iFrame = CreateFrame("Frame", "GladiminishIconFrame"..i, GladiusFrame)
	
	iFrame:ClearAllPoints()
	iFrame:SetPoint("TOPRIGHT", gladiusButton, "TOPLEFT", -DIST_TO_FRAME, 0)
	iFrame:SetPoint("BOTTOMRIGHT", gladiusButton, "BOTTOMLEFT", -DIST_TO_FRAME, 0)
	iFrame:SetWidth(1)
	
	iFrame:SetScript("OnUpdate", iconUpdate)
	iFrame.numIcons = 0
	iFrame.icons = {}
	
	return iFrame
end

-- Update existing iconframe
function Gladiminish:UpdateIconFrame(i)
	local gladiusButton = "GladiusButton"..i
	local iFrame = self.iFrames["arena"..i]
	
	iFrame:ClearAllPoints()
	iFrame:SetPoint("TOPRIGHT", gladiusButton, "TOPLEFT", -DIST_TO_FRAME, 0)
	iFrame:SetPoint("BOTTOMRIGHT", gladiusButton, "BOTTOMLEFT", -DIST_TO_FRAME, 0)
end

-- Return color based on DR factor - soon to be replaced
function Gladiminish:GetColor(factor)
	if factor == 0.5 then
		return 1, 1, 0, 1
	elseif factor == 0.25 then
		return 1, 0.5, 0, 1
	else
		return 1, 0, 0, 1
	end
end

-- Create an icon
function Gladiminish:CreateIcon(parent, category, spellID, factor, duration)
	local icon = CreateFrame("Frame", nil, parent)
	icon.category = category
	icon.spellID = spellID
	icon.factor = factor
	icon.timeLeft = duration
	icon.fixed = nil
	
	icon:ClearAllPoints()
	icon:SetPoint("TOPRIGHT", 0, 0)
	
	local size = parent:GetHeight()
	icon:SetWidth(size)
	icon:SetHeight(size)
	
	-- texture
	local _, _, path = GetSpellInfo(spellID)
	icon.texture = icon:CreateTexture(nil, "ARTWORK")
	icon.texture:SetTexture(path)
	icon.texture:SetAllPoints(icon)
	
	local font = GameFontNormalSmall:GetFont()	
	-- duration text
	icon.durationText = icon:CreateFontString(nil, "OVERLAY")
	icon.durationText:SetFont(font, FONT_SIZE) -- DB
	icon.durationText:SetShadowOffset(1, -1)
	icon.durationText:SetShadowColor(0, 0, 0, 1)
	icon.durationText:SetJustifyH("CENTER")
	icon.durationText:SetPoint("CENTER", 0, 0)
	
	icon.durationText:SetTextColor(self:GetColor(factor))
	
	return icon
end

-- Update an existing icon
function Gladiminish:RefreshIcon(icon, parent, category, spellID, factor, duration)
	icon.spellID = spellID
	icon.category = category
	icon.factor = factor
	icon.timeLeft = duration
	icon.fixed = nil
	
	icon:SetParent(parent)
	icon:ClearAllPoints()
	icon:SetPoint("TOPRIGHT", 0, 0)
	
	local size = parent:GetHeight()
	icon:SetWidth(size)
	icon:SetHeight(size)
	
	local _, _, path = GetSpellInfo(spellID)
	icon.texture:SetTexture(path)
	icon.texture:ClearAllPoints()
	icon.texture:SetAllPoints(icon)
	icon.durationText:SetTextColor(self:GetColor(factor))
	
	icon.durationText:ClearAllPoints()
	icon.durationText:SetPoint("CENTER", 0, 0)
end